/*!
  spica javascript libraries - fader.js
  == written by Takuya Otani <takuya.otani@gmail.com> ===
  == Copyright (C) 2006 SimpleBoxes/SerendipityNZ Ltd. ==
*/

// [guideline]
// -- http://serennz.sakura.ne.jp/guidelines/js_en_000.txt
// -- http://serennz.sakura.ne.jp/guidelines/js_ja_000.txt

// ver 0.01 [2008/12/05] fixed a bug in ObjectResizer
// ver 0.00 [2008/11/01] the first attempt

Spica.Library.require('color','element');
Spica.ColorShifter = function(target,option)
{
	var self = this;
	if (!target) return;
	self.background = true;
	self.duration = 100;
	self.max = 10;
	// probably the following variables are used internally.
	self.object = target;
	self.num = 0;
	self.diff = null;
	self.dest = new Spica.Color( (option.dest) ? option.dest : '#fff' );
	self.color = new Spica.Color( (option.init) ? option.init : '#f00' );
	return self;
}
Spica.ColorShifter.prototype = {
	onFinished : function()
	{ // default handler - should be overridden
		alert('done');
	},
	start : function(firstWait)
	{
		var self = this;
		self.diff = (new Spica.Color(self.dest)).subtract(self.color).divide(self.max);
		(self.background) 
			? self.color.fill(self.object) 
			: self.color.tint(self.object);
		window.setTimeout(function() { self._updateColor() },firstWait);
	},
	_updateColor : function()
	{
		var self = this;
		self.color = self.color.add(self.diff);
		(self.background) 
			? self.color.fill(self.object) 
			: self.color.tint(self.object);
		self.num++;
		if (self.num < self.max)
		{
			window.setTimeout(function() { self._updateColor() },self.duration);
			return;
		}
		self.dest.fill(self.object);
		self.onFinished();
	}
};
Spica.ObjectResizer = function(target,option)
{
	var self = this;
	if (!target) return;
	self.object = target;
	self.targetSize = {
		w: (option.width != undefined)  ? option.width  : Spica.DOM.getWidth(self.object),
		h: (option.height != undefined) ? option.height : Spica.DOM.getHeight(self.object)
	};
	self.diff = {
		x: self.targetSize.w - Spica.DOM.getWidth(self.object),
		y: self.targetSize.h - Spica.DOM.getHeight(self.object)
	};
	self.duration = 100;
	self.num = 0;
	self.max = 10;
	self.buf = Spica.DOM.getHeight(self.object);
}
Spica.ObjectResizer.prototype = {
	onFinished : function()
	{ // default handler - should be overridden
		alert('done');
	},
	start : function(firstWait)
	{
		var self = this;
		if (self.diff.x != 0 || self.diff.y != 0)
			window.setTimeout(function() { self._updateSize() },firstWait);
		else
			self.onFinished();
	},
	_updateSize : function()
	{
		var self = this;
		var width = Spica.DOM.getWidth(self.object);
		width += parseInt(self.diff.x / self.max);
		self.object.style.width = width + 'px';
		var height = Spica.DOM.getHeight(self.object);
		height += parseInt(self.diff.y / self.max);
		self.object.style.height = height + 'px';
		self.num++;
		if (self.num < self.max)
		{
			window.setTimeout(function() { self._updateSize() },self.duration);
			return;
		}
		self.onFinished();
	}
};
